import logging
logger = logging.getLogger(__name__)

import operator

from enthought.interpolate import linear
from enthought.numerical_modeling.numeric_context.api import NumericContext
from enthought.traits.api import Bool, Instance
from enthought.util.equivalence import Equivalence
from enthought.util.sequence import any, union

# TODO Ability to name groups?

class GroupingPrototype(NumericContext):

    # An equivalence relation on log names specifying which are grouped
    # together.
    groups = Instance(Equivalence)
    _groups_default = lambda self: Equivalence()

    # An equivalence relation on index names specifying which are correlated.
    index_correlations = Instance(Equivalence)
    _index_correlations_default = lambda self: Equivalence()

    # By default, add incoming arrays to a group with arrays of the same length
    # as the incoming array.
    auto_group = Bool(True)

    # Whether to raise an error when an automatic group assignment is ambiguous
    error_on_ambiguous_auto_grouping = Bool(True)

    ### NumericContext interface ##############################################

    def _context_modified_fired(self, e):

        # If parent classes start listening for 'context_modified', we don't
        # want to break them
        try:
            f = super(GroupingPrototype, self)._context_modified_fired
        except AttributeError:
            pass
        else:
            f()

        # Remove grouping and correlation information about removed logs
        self.groups.remove(*e.removed)
        self.index_correlations.remove(*e.removed)

        # Automatic grouping?
        if self.auto_group:

            # Index the groups by size
            groups_by_size = {}
            for g in self.groups:
                assert len(set( len(self.get_dotted(x)) for x in g ))
                size = len(self.get_dotted(iter(g).next()))
                groups_by_size.setdefault(size, set()).add(g)

            # Each array not explicitly in a group is in its own implicit
            # group. (Keep in mind that incoming names are already in the
            # dictionary.)
            for name in (set(self.context_all_names) - union(self.groups) -
                         set(e.added)):
                x = self.get_dotted(name)
                if self.is_valid_array(x):
                    g = frozenset([name])
                    size = len(x)
                    groups_by_size.setdefault(size, set()).add(g)

            # Find the auto-group for each new element based on the element's
            # size
            for a in e.added:
                gs = groups_by_size.get(len(self.get_dotted(a)))
                if gs is not None:
                    assert len(gs) != 0
                    if len(gs) == 1:
                        [g] = list(gs)
                        self.groups.equate(a, iter(g).next())
                    elif len(gs) > 1:
                        msg = (
                            'Ambiguous automatic group assignment: '
                            '%s fits in any of %s' % (a, gs)
                        )
                        if self.error_on_ambiguous_auto_grouping:
                            raise ValueError(msg)
                        else:
                            logger.warn(msg)

    ### GroupingPrototype interface ###########################################

    def indexed_like(self, log, example, interpolate=linear):

        # If 'log' and 'example' are in the same group, then 'log' is already
        # indexed like 'example'
        if self.groups.equivalent(log, example):
            return self.get_dotted(log)

        # We want to find a pair of correlated indices, one from 'from_indices'
        # and one from 'to_indices'. Further, if an index was given for
        # 'example', then don't consider any other indices in 'to_indices'.
        #
        # Then we resample 'a' if we can find such a pair.

        from_indices = self._indices_for(log)
        if self._is_index(example):
            to_indices = [example]
        else:
            to_indices = self._indices_for(example)

        for j in to_indices:
            for i in from_indices:
                if self.index_correlations.equivalent(i, j):
                    return self._resample(self.get_dotted(i),
                                          self.get_dotted(log),
                                          self.get_dotted(j),
                                          interpolate)
        else:
            raise ValueError('No conversion from %r to %r' % (log, example))

    ### GroupingPrototype protected interface #################################

    def _is_index(self, name):
        return any( name in c for c in self.index_correlations )

    def _indices_for(self, name):
        return filter(self._is_index, self.groups.get_class(name))

    # TODO Rudimentary; factor out resampling code from cp.log.log (#2668)
    def _resample(self, from_index, log, to_index, interpolate):
        return interpolate(from_index, log, to_index)
