#-----------------------------------------------------------------------------
#
#  Copyright (c) 2006 by Enthought, Inc.
#  All rights reserved.
#
#  Author: Bryce Hendrix <bhendrix@enthought.com>
#
#-----------------------------------------------------------------------------

import logging

# ETS imports
from enthought.traits.api import Instance, Dict, Property
from enthought.units.geo_units import fractional, ppg, psi
from enthought.units.length import ft
from enthought.units.api import convert
from enthought.numerical_modeling.numeric_context.api import NumericContext


# Local imports.
from log_context import LogContext


#special conversions
ppg_to_psi = 0.0519 * ft
ppg_to_sg = 0.119826427 * fractional
psi_to_ppg = 1/ppg_to_psi
psi_to_sg = ppg_to_sg * (psi/ppg)
sg_to_psi = (1/psi_to_ppg)*psi/psi.value
sg_to_ppg = (1/ppg_to_sg) * ppg/ppg.value

# Global logger
logger = logging.getLogger(__name__)

class LogUnitConversionContext(LogContext):
    """ WARNING: once this instance is constructed, it will feed changes back
        to the context it was constructed from, but will not pull changes from
        that context.


    """

    conversion_map = Dict()
    context = Instance(NumericContext)
    context_name = Property

    def __init__(self, context, *args, **kw):
        super(LogUnitConversionContext, self).__init__(*args, **kw)
        self.context = context
        for k,v in context.items():
            self[k] = v

    def dispose(self):
        self.context = None
	self.conversion_map.clear()

	super(LogUnitConversionContext, self).dispose()

    def set_units(self, key, units):
        self.conversion_map[key] = units

    def get_dotted(self, key):
        value = super(LogUnitConversionContext, self).get_dotted(key)
        if self.conversion_map.has_key(key):
            units = self.conversion_map[key]
            return self._convert_to(value, units)
        return value

    def lookup(self, key):
        return self.context.lookup()

    def get_index(self):
        return self.context.get_index()

    def get_index_name(self):
        return self.context.get_index_name()

    def set_index(self, name):
        self.context.set_index(name)
        return

    def edit_data(self, name, data):
        return self.context.edit_data(name, data)

    def edit_index_by_shift(self, name, data, shift):
        """ Changing index as well as data during depth shift of a log
        """
        return self.context.edit_index_by_shift(name, data, shift)

    def clone_traits(self, traits = None, memo = None, copy = None, **metadata):
        """ Cloning the numeric context is a bad idea and hardly ever works as
        expected.

        TODO: remove this method in the future when cloning works better

        """

        logger.warn("Cloning LogUnitConversionContext disabled. Operating " + \
                    "on the real context, not a copy")

        return self
    
    def _get_context_name(self):
        return self.context.context_name
    
    def _set_context_name(self, new_name):
        self.context.context_name = new_name

    def __getitem__(self, key):
        value = self.context[key]
        if self.conversion_map.has_key(key):
            units = self.conversion_map[key]

            # Geo units have some special cases, where its not really
            # a unit conversion (psi->ppg, I'm looking at you).
            if units == ppg:
                new_value = value.copy()
                if new_value.units == psi:
                    depth = self.get_index()/self.get_index().units.value
                    new_value = new_value * psi_to_ppg.value / depth
                    new_value.units = ppg
                elif new_value.units == fractional:
                    new_value = new_value * psi_to_sg.value
                    new_value.units = ppg
                try:
                    setattr(new_value, 'name', value.name)
                except:
                    pass                
                return new_value

            elif units == psi:
                new_value = value.copy()
                if new_value.units == ppg:
                    depth = self.get_index()/self.get_index().units.value
                    new_value = new_value * ppg_to_psi.value * depth
                    new_value.units = psi
                if new_value.units == fractional:
                    new_value = new_value * sg_to_psi.value
                    new_value.units = psi
                try:
                    setattr(new_value, 'name', value.name)
                except:
                    pass                
                return new_value

            elif units == fractional:
                new_value = value.copy()
                if new_value.units == ppg:
                    new_value *= ppg_to_sg.value
                    new_value.units = fractional
                if new_value.units == psi:
                    new_value *= psi_to_sg.value
                    new_value.units = fractional
                try:
                    setattr(new_value, 'name', value.name)
                except:
                    pass                
                return new_value

            new_value = self._convert_to(value, units)
            try:
                setattr(new_value, 'name', value.name)
            except:
                pass
            return new_value 
            
        return value

    def __setitem__(self, key, value):
        super(LogUnitConversionContext, self).__setitem__(key, value)
        if self.conversion_map.has_key(key):
            units = self.conversion_map[key]
            value = self._convert_from(value, units)
        self.context[key] = value

    def _convert_to(self, original, units):
        if isinstance(units, float):
            new = original.copy() * units
            new.units = fractional
        else:
            new = convert(original.copy(), original.units, units)
            new.units = units

        return new

    def _convert_from(self, original, units):
        if isinstance(units, float):
            new = original.copy() * units
            new.units = fractional
        else:
            new = convert(original.copy(), units, original.units)
            new.units = units

        return new

