#-----------------------------------------------------------------------------
#
#  Copyright (c) 2008 by Enthought, Inc.
#  All rights reserved.
#
#-----------------------------------------------------------------------------

""" UI for displaying spreadsheet view of a log-context 
"""

# Standard library imports
import numpy

# Enthought library imports
from enthought.numerical_modeling.units.unit_array import UnitArray
from enthought.traits.api import HasTraits, List, Str, Array, Bool
from enthought.traits.ui.api import View, Item
from enthought.traits.ui.menu import NoButtons
from enthought.traits.ui.wx.tabular_editor import \
     TabularEditor, TabularAdapter
from enthought.units.api import unit_manager

# Geo imports
from geo.data.api import LogContext
    
#-----------------------------------------------------------------------------
#  SpreadsheetUI class
#-----------------------------------------------------------------------------

class SpreadsheetUI(HasTraits):
    """ UI for displaying spreadsheet view of a log context
    """

    # Processed input data
    unit_arrays = List(UnitArray)
    names = List(Str)

    # View details
    data = Array
    title_name = Str('')
    trim_title = Bool(False)
    
    def __init__(self, arrays, names=None, **traits):
        super(SpreadsheetUI, self).__init__(**traits)
        final_unit_arrays, final_names = [], []

        # Check if input is a list of arrays
        if isinstance(arrays, list) and isinstance(names, list) and \
               len(arrays) == len(names):
            for i, array in enumerate(arrays):
                # Remove duplicate logs; and retain just valid data.
                if isinstance(array, UnitArray) and \
                   isinstance(names[i], basestring) and \
                   not names[i] in final_names:
                    final_unit_arrays.append(array)
                    final_names.append(names[i])
            self.title_name = ', '.join(final_names)

        # Check if it is a log-context
        elif isinstance(arrays, LogContext):
            for k, v in arrays.items():
                final_unit_arrays.append(v)
                final_names.append(k)
            self.title_name = arrays.context_name

        # Check if it is a single log
        elif isinstance(arrays, UnitArray) and isinstance(names, basestring):
            final_unit_arrays.append(arrays)
            final_names.append(names)
            self.title_name = names

        self.unit_arrays = final_unit_arrays
        self.names = final_names

        # Create data for view.
        self.create_data()
        
    def trait_view(self, name=None, view_element=None):
        title = 'Spreadsheet view of %s' % self.title_name
        if self.trim_title and len(title) > 18:
            title = '%s...' % title[:15]
            
        columns = [(name, i) for i,name in enumerate(self.names)]
        
        tabular_editor = TabularEditor(
            adapter = TabularAdapter(columns = columns))
        
        height = min(400, 10*self.data.shape[0])
        width = min(800, 100*len(columns))

        return View(Item('data', editor = tabular_editor,
                         show_label = False),
                    title = title,
                    width = width,
                    height = height,
                    resizable = True,
                    buttons = NoButtons)
    
    def create_data(self):
        """ Create data from list of arrays
        """

        data = []
        for i, log in enumerate(self.unit_arrays):
            if log.dtype.char != 'O' and log.dtype.char != 'S':
                # If its not string data, convert to the correct units
                # and replace NaN's with 'Null'
                new_log = unit_manager.convert(log)
                final_log = numpy.where(numpy.isfinite(new_log),
                                        new_log, 'Null')
            else:
                final_log = list(log)

            # Add units to the column titles.
            if log.units != None:
                self.names[i] = '%s (%s)' % (self.names[i], log.units.label)
            else:
                self.names[i] = '%s (None)' % self.names[i]

            data.append(final_log)

        # Add an index column
        if len(data):
            data.insert(0, numpy.arange(1, len(data[0])+1))
            self.names.insert(0, 'index')
            
        self.data = numpy.array(data).T
        return
    
### EOF -----------------------------------------------------------------------
