#-----------------------------------------------------------------------------
#
#  Copyright (c) 2008 by Enthought, Inc.
#  All rights reserved.
#
#-----------------------------------------------------------------------------

""" UI for displaying summary of a log-context 
"""

# Standard library imports
import numpy

# Enthought library imports
from enthought.traits.api import HasTraits, Instance, List, on_trait_change
from enthought.traits.ui.api import View, Item
from enthought.traits.ui.menu import NoButtons
from enthought.traits.ui.wx.tabular_editor import \
     TabularEditor, TabularAdapter
from enthought.units.api import unit_manager
from enthought.util.safe_array_opts import \
     safe_len, safe_min, safe_max, safe_mean, safe_flat

# Geo library imports
from geo.data.api import LogContext
    
#-----------------------------------------------------------------------------
#  SummaryUI class
#-----------------------------------------------------------------------------

class SummaryUI(HasTraits):
    """ UI for displaying summary of a log context
    """
    
    unit_arrays = Instance(LogContext)
    data = List
        
    def trait_view(self, name=None, view_element=None):
        title = 'Summary of %s' % self.unit_arrays.context_name
        
        columns = [('Log Name', 0), ('Measure of', 1), ('Units', 2),
               ('Min', 3), ('Max',4), ('Mean', 5),
               ('Samples', 6), ('Nulls', 7), ('Vectors', 8)]
        
        tabular_editor = TabularEditor(
            adapter = TabularAdapter(columns = columns))
        
        return View(Item('data', editor = tabular_editor,
                         show_label = False),
                    title = title,
                    height = 300,
                    width = 700,
                    resizable = True,
                    buttons = NoButtons)

    @on_trait_change('unit_arrays')
    def create_data(self):
        """ Create data from the log-context
        """

        self.data = []
        
        for name in self.unit_arrays.context_names:
            log = unit_manager.convert(self.unit_arrays.get_dotted(name))
            unit_family = unit_manager.get_family_name(name.lower())
            if unit_family == 'unknown':
                unit_family = unit_manager.get_family_name_for_value(log.units)
            llen = safe_len(log)
            item = [name, unit_family, str(log.units.label)]
            
            if isinstance(log[0], basestring):
                # Text Log
                condition = [value == '' for value in log]
                nans = numpy.sum(numpy.compress(condition, numpy.arange(llen)))
                item.extend(['n/a', 'n/a', 'n/a', llen, 1 , nans])
            else:
                lshape = numpy.shape(log)
                flatlog = safe_flat(log)

                if len(lshape) < 2:
                    nvecs, nsmps = 1, llen
                else:
                    nvecs, nsmps = lshape
                    
                # Strip any NaNs, then get attrs
                idxs = numpy.nonzero(numpy.isfinite(flatlog))[0]

                # Check for all NaNs
                if safe_len(idxs) > 0:
                    newlog = numpy.take(flatlog, idxs)
                    noks = len(newlog)
                    lmin = float(numpy.min(newlog))
                    lmax = float(numpy.max(newlog))

                    # mean(x) fails if x is a constant array
                    if lmin == lmax:
                        lmean = lmin
                    else:
                        lmean = float(numpy.mean(newlog))

                    lmin = '%.3f' % lmin
                    lmean = '%.3f' % lmean
                    lmax = '%.3f' % lmax
                else:
                    # Entire log is NaNs
                    lmean = lmin = lmax = 'NaN'
                    noks = 0
                    
                item.extend([lmin, lmax, lmean,
                             nsmps, safe_len(flatlog)-noks, nvecs])
               
            self.data.append(item)
            
        return

### EOF -----------------------------------------------------------------------
