from copy import copy
from numpy import array

from enthought.interpolate import linear

def interpolate_mask(x, mask, interpolate=linear, boundary=0):
    ''' Interpolate regions in array 'x' specified by 'mask'.

        Interpolate using function 'interpolate' whose signature is consistent
        with those in enthought.interpolate (FIXME). When 'mask' includes the
        first or last element, assume an extra value 'boundary' on either end
        of the array.

        >>> interpolate_mask(array([5,2,3,1,8]), [True,False,True,True,False])
        array([1, 2, 4, 6, 8])

        >>> interpolate_mask(array([5,2,3,1,8]), [True,True,False,True,True])
        array([1, 2, 3, 2, 1])

        >>> interpolate_mask(array([5,2,3,1,8]), [True,True,True,True,True])
        array([0, 0, 0, 0, 0])

        >>> interpolate_mask(array([5.,2.,3.]), [True,False,True], boundary=1)
        array([ 1.5,  2. ,  1.5])
    '''
    x = copy(x)

    # TODO Handle the various types of masks

    # Interpolate each masked region
    for a,b in _regions(mask):

        # Find the two values to interpolate between
        if a == 0:
            left = boundary
        else:
            left = x[a-1]
        if b == len(x):
            right = boundary
        else:
            right = x[b]

        # Interpolate, slicing the endpoints of the interpolation: they are
        # either already in the array, or they are the boundary values and
        # don't belong in the array anyway
        x[a:b] = interpolate([a-1, b], [left, right], range(a-1, b+1))[1:-1]

    return x

def _regions(mask):
    ''' Generate pairs of endpoints of contiguous sections of mask.

        >>> list(_regions([True, True, False, True, False, False, True]))
        [(0, 2), (3, 4), (6, 7)]

        >>> from numpy import arange, pi, sin
        >>> list(_regions(sin(arange(0, 2*pi, pi/8)) < 1e-6))
        [(0, 1), (8, 16)]
    '''
    in_region = False
    for i,b in enumerate(mask):
        if b and not in_region:
            in_region = True
            start = i
        if in_region and not b:
            in_region = False
            yield (start, i)
    if in_region:
        yield (start, len(mask))


################################################################################
# Testing
################################################################################

import unittest
from enthought.testing.doctest_tools import doctest_for_module
import interpolate

class InterpolateDocTest(doctest_for_module(interpolate)):
    pass

if __name__ == '__main__':
    import sys
    unittest.main(argv=sys.argv)
