""" Class for reading in information from a Las Well Log file.

    See las_format.txt in this directory for more information on
    the LAS file format.

    The module can be run at the command line to print out
    a simple header report about a las file.

    Example:
        $ python las_header_reader.py tests/sample.las

    fixme: Documentation... What LAS versions does it handle?
    fixme: Should be handed a logger handle to report errors.
"""

# Standard library imports
import logging
from numpy import NaN

# Enthought library imports
from enthought.traits.api import List, Property

# Geo library imports
from geo.io.parameter_info_item import ParameterInfoItem
from geo.io.log_file_header_base import LogFileHeaderBase


# Create a logger instance for this module.
logger = logging.getLogger(__name__)


class LasHeader(LogFileHeaderBase):
    """ Reads a LAS header to get information about the file.

        This class reads the data out of a las well log file header and
        keeps track of the various attributes found there.

        Each line of the header either defines a new section, a comment, or a
        parameter of some sort.  This reader skips comments and only reads the
        parameters out of the Well, Curve, Parameter, and Other sections of
        the file.   It is very minimalistic in its attempt to convert data to
        specific data types.  Instead, it holds the information in general
        purpose LasInfoItem records that simply store information such as the
        name, units, data, description, etc. for attributes in string form.
        This is intentional as it makes the class useful for loading las files
        into a wide range of formats.

        The reading of a las file header is implemented separately from
        reading the entire file, because it provides information to the user
        that they may use to change how the rest of the data in the file is
        read/interpreted.

        The class is not intended to be a comprehensive implementation of
        the LAS standard.  Instead it implements features to read much of
        the data in the header and makes the most important data easily
        accessible.

    """

    #### 'LogFileHeaderBase' interface #########################################

    # Name of the well.
    well_name = Property(depends_on='well_info')

    # Floating point value used to indicate Nulls in data.
    null_value = Property


    ### 'LasHeader' interface. #################################################

    # List of LasInfoItem objects from the "Well" section of the las data.
    well_info = List(ParameterInfoItem)

    # List of LasInfoItem objects from the "Parameter" section of the las data.
    parameter_info = List(ParameterInfoItem)

    # List of LasInfoItem objects found in the "Other" section of the las data.
    other_info = List(ParameterInfoItem)


    ############################################################################
    # 'LasHeader' interface.
    ############################################################################


    ### Private static methods #################################################

    @staticmethod
    def _load_header(file):
        """ Return a list of all the lines of the las header.

            Reads header records until it comes to the ~A or an empty line,
            which signifies the end of the header and the start of the data.
        """

        records = []

        last_section = False

        # loop until we get to the end of the header
        for line in file:
            if last_section and not line.startswith('#'):
                # we're done!
                break
            elif line[:2] == "~A" or line == '':
                # We're at the beginning of the last section.
                # signal to stop grabbing lines once this section is finished.
                last_section = True
                records.append(line)
            else:
                # this line is part of the header -- just append it.
                records.append(line)

        return records


    ############################################################################
    # Protected interface.
    ############################################################################

    #### Property get/set methods ##############################################

    def _get_well_name(self):
        """ Return the well name if it is specified in the "Well" Section.
            Otherwise, return an empty string.
        """

        # Extract ParameterInfoItems referring to the well
        info_item = [x for x in self.well_info if x.name.lower() == 'well' or \
                     x.name.lower() == 'wn']

        # Extract the well names
        name = [i.data for i in info_item]
        if len(name):
            well_name = name[0]
            if well_name != '' and isinstance(well_name, basestring):
                # Sometimes the strings are in utf-encoding, and one would
                # like to get simplistic ascii representation. Since these cases
                # are rare, the decoding part is avoided here. Ascii represent-
                # ation is a must for bringing up the traits view etc. without
                # any problems
                well_name = repr(well_name)
                result = well_name[1:-1]
                return result

            # At times, the well name is extracted into the units part of
            # ParameterInfoItem. Hence, a final look in there...
            name = [i.units for i in info_item]
            if name:
                well_name = name[0]
                if well_name != '' and isinstance(well_name, basestring):
                    # Same comment as abovementioned on ascii representations
                    well_name = repr(well_name)
                    result = well_name[1:-1]
                    return result

        result = "Well"
        return result


    def _get_null_value(self):
        """ Return the well name if it is specified in the "Well" Section.
            Otherwise, return an empty string.
        """
        name = [x.data for x in self.well_info if x.name.lower() == "null"]
        if name:
            return float(name[0])
        else:
            return NaN


    def _well_name_changed(self, old, new):
        logger.debug('Detected well_name changed from [%s] to [%s]', old, new)

        return


    #### LAS Processing methods ################################################
    # These methods do the heavy lifting of actually parsing the header.
    ############################################################################


    def _process_header(self, records):
       """ Process the lines of the header, putting the records into
           the appropriate sections (well_info, log_info, etc.).
       """

       for line, record in enumerate(records):

          # load lines beginning with one of the mnemonics
          key = record[:2]

          if key == '~V': # Version section of file.
              # Load lines from the ~Version section of the LAS file.
              # contains version and wrap mode information
              #self._load_las_information(records, line+1)
              pass
          elif key == '~W': # Well information
              self.well_info = self._load_las_information(records, line+1)
          elif key == '~C': # Curve Infomation
              self.log_info = self._load_las_information(records, line+1)
          elif key == '~P': # Parameter Information
              info = self._load_las_information(records, line+1)
              self.parameter_info = info
          elif key == '~O': # Other Information
              self.other_info = self._load_las_information(records, line+1)
          else:
              pass

       return


    def _load_las_information(self, records, line):
       """ Load a line into a list of ParameterInfoItem objects
       """

       # list of las info objects found in this section
       info_objects = []

       # loop until the end of the section, in this case indicated
       # by the marker for the beginning of the data section
       first_is_index = True

       while not records[line].startswith("~"):
           # skip comments and blank lines
           if not records[line].startswith("#") and \
                  len(records[line].strip()) > 0:
               # build a LogInfo object from a string.
               info = _parameter_info_item_from_las_string(records[line])
               info.is_index = first_is_index
               first_is_index = False
               info_objects.append(info)
           line += 1

       return info_objects

################################################################################
# Helper Functions
################################################################################

def _parameter_info_item_from_las_string(string):
    """ Factory to generate ParameterInfoItem from a string in LAS format.

        This method parses a line from a las header and returns a
        ParameterInfoItem object that represents it.
    """

    # everything before the first '.' is the MNEM
    name, remainder = string.split( '.', 1 )
    name = name.strip()

    # Next come the units.
    units, remainder = remainder.split( ' ', 1 )
    units = units.strip()

    # finally extract [DATA|VALUE|API CODE]
    data, remainder = remainder.split(':', 1)
    data = data.strip()

    # anything after the ':' is the 'description'
    description = remainder.strip()

    return ParameterInfoItem(name=name, description=description,
                             units=units, data=data)


################################################################################
# Main function (for testing)
################################################################################

def main(argv=None):
    """ usage: las_header_reader.py las_file.las
    """

    # Standard library imports.
    from optparse import OptionParser

    ### Option parsing section #################################################
    usage = ('%prog file.las\n'
             'print out header information read from a las file')

    parser = OptionParser(usage=usage)

    if argv is None:
        import sys
        argv = sys.argv[1:]

    options, args = parser.parse_args(argv)
    if len(args) != 1:
        parser.print_usage()
        raise SystemExit

    file_name = argv[0]
    ### Read haeder and parse it #############################################

    file = open(file_name,'r')

    las = LasHeader.from_file(file)

    ### Write out simple report ##############################################

    print 'file:', file_name
    print
    print "well info:"
    for obj in las.well_info:
        print obj

    print "parameter info:"
    for obj in las.parameter_info:
        print obj

    print "logs:"
    for obj in las.log_info:
        print obj


if __name__ == "__main__":
    main()

#### EOF ######################################################################

