#-----------------------------------------------------------------------------
#
#  Copyright (c) 2007 by Enthought, Inc.
#  All rights reserved.
#
#-----------------------------------------------------------------------------

""" Class file for LasWriter
"""

# Standard imports
import time
import logging
import numpy
from scipy import isnan, array, transpose, putmask, ravel
from string import join

# ETS lib imports
from enthought.units.unit_manager import unit_manager
from enthought.numerical_modeling.units.unit_array import UnitArray

# Geo lib imports
from geo.data.log_context import LogContext

# Local imports
from write_array import write_array

# Creating a logger
logger = logging.getLogger(__name__)


#-------------------------------------------------------------------------------
#   LasWriter Class
#-------------------------------------------------------------------------------

class LasWriter:
    """ Class for writing Las files given a log_context

    """

    def __init__(self, ls, filename, contextname = 'LogSuite' ):
        """ Constructor for a Las Writer

            Parameters:
            -----------
            filename: Str
                Name of the file to be written
            contextname: Str
                Name of the context to be written
            ls: Log Context
                Logsuite of the well to be written

        """

        if isinstance(ls, LogContext):
            self.ls = ls
        else:
            logger.error('Constructor: Input is not a LogContext')
            self.ls = None

        self.filename = filename
        self.contextname = contextname
        self._write_header()

        return

    def _write_header(self):
        """ Writing header of a LAS file
        """

        self._fh = open(self.filename, 'w')

        self._save_version_information()
        self._save_well_information()
        self._save_curve_information()
        self._save_parameters()
        self._save_other_information()
        self._write_data()

        self._fh.close()
        return

    def _save_version_information(self):
        """ Saving version information of the data in the header
        """

        commented_dotted_line = ''.join(("#",'-'*79))

        self._writeln(commented_dotted_line)
        self._writeln("#")
        self._writeln(''.join(("#   Created by    : CPLab (Developed by ",
                      "Enthought Inc.) Log Export")))
        self._writeln("#   Created on    : %s" % \
                      time.asctime(time.localtime(time.time())))
        self._writeln("#   %s %s" % ("Log Suite Name:", self.contextname))
        self._writeln("#")
        self._writeln(commented_dotted_line)

        self._writeln("~Version information")
        self._write("VERS", value="2.0", descr="CWLS LAS  -  VERSION 2.0")

        # The LAS standard has deprecated support for line wrapping
        # so I see no reason for us to support it
        self._write("WRAP", value="NO", descr="Single line per depth step")
        self._write("DLM", value="SPACE", descr="Column Data Section Delimiter")

        return

    def _save_well_information(self):
        """ Saving well information: name, location, and time of measurement in
            the file header

        """

        self._writeln(''.join(('~Well information',' '*61)))

        if self.ls is not None:
            data = self.ls.get_index()
            wname = "%s" % (self.contextname)

            strt = data[0]
            stop = data[-1]
            step = data[1] - data[0]
        else:
            logger.error('LasWriter: Empty LogContext input')
            data = None
            wname = None
            strt = None
            stop = None
            step = None

        if isinstance(data, UnitArray):
            if data.units is not None:
                unit = data.units.label
            else:
                unit = None
        else:
            logger.error('Save well information: Data is not a unit_array')
            unit = None

        self._write(''.join(('#','MNEM.UNIT','       ','Data Type','    ',
                             'Information',' '*37)))
        self._write(''.join(('#','---------','     ','-----------','    ',
                             '-------------------------------',' '*17)))

        self._write_with_precision("STRT", unit, strt, "First depth/time")
        self._write_with_precision("STOP", unit, stop, "Last depth/time")
        step_description = ''.join(("Depth/time increment - determined from ",
                                    "first two data points"))
        self._write_with_precision("STEP", unit, step, step_description)
        self._write("NULL", "", -999.25, "Null values")   # as a parameter?
        self._write("COMP"  "", "COMPANY", "")
        self._write("WELL"  "", wname, "")
        self._write("FLD "  "", "FIELD", "")
        self._write("LOC "  "", "LOCATION", "")
        self._write("PROV"  "", "PROVINCE", "")
        self._write("SRVC"  "", "SERVICE COMPANY", "")
        self._write("CTRY"  "", "COUNTRY", "")
        self._write("DATE"  "", "LOG DATE", "")
        self._write("UWI "  "", "UNIQUE WELL ID", "")
        self._write("LATI"  "", "LATITUDE", "")
        self._write("LONG"  "", "LONGITUDE", "")

        return

    def _save_curve_information(self):
        """ Saving curve information in the header of the LAS file

        """

        if self.ls is not None:
            log_names = self.ls.context_names
            mnem_size = max( [len(name) for name in log_names] + [5] )
        else:
            logger.error('Save curve information: Empty LogContext input')
            mnem_size = 0

        self._writeln(join(("~Curve information"," "*60)))
        self._writeln(join(("#MNEM.UNIT", " "*(mnem_size-5+11),
                            "API Code      Curve Description", " "*31)))
        self._writeln(join(("#---------", " "*(mnem_size-5+11), "-----------",
                            "    -------------------------------"," "*17)))

        # output for the depth units
        if self.ls is not None:
            index_data = self.ls.get_index()
            index_log_name = self.ls.get_index_name()

            units = index_data.units
            if units is None:
                units = unit_manager.default_units_for(index_log_name)

                if units is None:
                    ufamily = unit_manager.get_family_name(index_log_name)
                    units = unit_manager.default_units_for(ufamily)

            # Convert '.' to '/' in names because '.' is a special token
            safe_index_name = index_log_name.replace('.', '/')
            if isinstance(index_data, UnitArray) and units is not None:
                self._write(safe_index_name, units.label, "", "1 TVD - ",
                            mnem_size)
            else:
                self._write(safe_index_name, 'Unknown', "", "1 TVD - ",
                            mnem_size)

            log_num = 2

            for log_name in log_names:
                if log_name != index_log_name:
                    log = self.ls.get_dotted(log_name)
                    units = log.units
                    if units is None:
                        units = unit_manager.default_units_for(log_name)

                        if units is None:
                            ufamily = unit_manager.get_family_name(log_name)
                            units = unit_manager.default_units_for(ufamily)

                    if units is not None and units.label is None:
                        ufamily = unit_manager.get_family_name_for_value(units)
                        units = unit_manager.default_units_for(ufamily)
                        
                    # Convert '.' to '/' in names because '.' is a special token
                    log_name = log_name.replace('.', '/')

                    if isinstance(log, UnitArray) and units is not None:
                        self._write(log_name, units.label, "",
                                    join((str(log_num)," ", log_name, " - ")),
                                    mnem_size)
                    else:
                        self._write(log_name, 'Unknown', "",
                                    join((str(log_num), " ", log_name, " - ")),
                                    mnem_size)

                    log_num += 1
        else:
            logger.error('Save curve information: Empty LogContext input')

        return

    def _save_parameters(self):
        """ Saving parameter information block
        """

        self._writeln(''.join(("~Parameter information block"," "*50)))
        self._writeln(''.join(("#MNEM.UNIT","         ","Value      ",
                               "Description"," "*37)))
        self._writeln(''.join(("#---------","     ","-----------","    ",
                               "-------------------------------"," "*17)))

        return

    def _save_other_information(self):
        """ Saving other information
        """

        self._writeln(''.join(("~Other information"," "*60)))
        return

    def _write(self, mnem, unit="", value="", descr="", mnem_size=4):
        """ Writing using a file handler

        """

        # !! how to handle longer lines??
        format = "%" + str(mnem_size) + "s.%-10s%20s: %-20s\n"
        self._fh.write( format % (mnem, unit, value, descr))
        return

    def _write_with_precision(self, mnem, unit="", value="",
                              descr="", mnem_size=4):
        """ Write the value as a float with a precision of 3
        """

        format = "%" + str(mnem_size) + "s.%-10s%20.3f: %-20s\n"
        self._fh.write( format % (mnem, unit, value, descr))
        return

    def _writeln(self, record):
        """ Write a line of information
        """

        self._fh.write(record)
        self._fh.write("\n")
        return

    def _write_data(self):
        """ Writing data from a log_context
        """

        self._fh.write("~A")

        if isinstance(self.ls, LogContext):
            index_log_name = self.ls.get_index_name()
            self._fh.write("%10s" % index_log_name.replace('.', '/'))
            data_arrays = []
            data_arrays.append(self.ls.get_index())

            for log_name in self.ls.context_names:
                if log_name != index_log_name:
                    log = self.ls.get_dotted(log_name)
                    self._fh.write(" %11s" % log_name.replace('.', '/'))
                    data_arrays.append(log)
            self._fh.write("\n")

            data_arrays = array(data_arrays).astype(numpy.float32)

            # put individual logs in columns of the array.  The copy() makes
            # a fresh copy of the data laid out that is in C (row major)
            #ordering in memory.
            data_arrays = transpose(data_arrays).copy()

            # replace all NaN values with -999.25
            # NB: depth at .5 ft requires 4 decimal places(.1524m == .5ft)
            # in meters
            putmask(ravel(data_arrays), isnan(ravel(data_arrays)), -999.25)

            write_array(file=self._fh,ary=data_arrays, format = "%12.3f",
                        line_separator = "\n")
        else:
            logger.error('Write data: Empty LogContext input')

        return
