""" LogConversionInfoItem describes how to convert an input log into a log suite.

    This class is generally used as a configuration model in UIs to allow users
    to choose which logs they want to load out of a file, whether they are
    unit converted, etc.
"""

# Enthought library Imports
from enthought.traits.api import HasTraits, Str, Bool, Enum, Property
from enthought.units import convert
from enthought.units.unit import InvalidConversion
from enthought.units.unit_parser import unit_parser, UnableToParseUnits
from enthought.units.unit_manager import unit_manager

# Local imports
from util import find_duplicates

class LogConversionInfoItem(HasTraits):
    """ Describes how to convert data from a source to a log.

        An example conversion would be taking a log velp in m/s and renaming
        it to vp in ft/s.  It also keeps track of whether this log should
        be loaded or not and what the log is a "measure of" (its units_family).

        While this class primarily stores the information about how the
        log should be converted, it also has checks to ensure that conversion
        units are compatible, etc.
    """

    #### 'LasHeaderReader' interface #########################################

    # Should this log be loaded or not?
    load = Bool(True)

    # Name of curve in las file.
    input_name = Str

    # Units in las file.
    input_units = Str

    # Name to use for log once imported.
    desired_name = Str

    # String, such as "m/s", that can be parsed by the enthought.units.parser
    desired_units = Str

    # "measure of" family that describes what this log measures.
    unit_family = Str

    # Text description of the log such as "primary velocity in shale"
    description = Str

    # True if input_units string is parsable to a valid unit object.
    input_units_valid = Property

    # True if desired_units string is parsable to a valid unit object.
    desired_units_valid = Property

    # True if input_units and desired_units are compatible.
    units_compatible = Property

    # Status informs if conversion has a potential or fatal conversion
    # problem.  This could be incompatible las->desired units or units
    # that don't match the unit_family.
    # fixme: I am not as sure about the naming of this or the info
    #        it provides.
    units_status = Enum('normal', 'warning', 'error')

    ############################################################################
    # LogConversionInfoItem interface
    ############################################################################

    ### Static class methods ###################################################

    @staticmethod
    def cleanup_list(info):
        """ Various 'repairs' on a list of LogConversionInfoItems.

            Return the list of info items with the following changes made.

             - Remove spaces from any 'desired_name' entries.
             - Find any duplicate 'desired_name' entires and rename
               appropriately.  For example, if GR is found 3 times, they will
               be renamed GR, GR2, GR3.
             - If the desired output units are not parsable, replace with None.
               (not implemented yet).
        """

        # Cleanup the data.
        # fixme: refactor this to a more general location.

        # first remove spaces from desired names.
        for item in info:
            item.desired_name = item.desired_name.replace(' ', '')

        # Find duplicate desired names.
        names = [item.desired_name for item in info]
        duplicates = find_duplicates(names)
        for name in duplicates:
            # Ignore the first duplicate, we'll leave its name alone.
            indices = duplicates[name][1:]

            # For each duplicate name, change the desired name to x1, x2, etc.
            for i, index in enumerate(indices):
                info[index].desired_name = "%s%d" % (name, i+2)

        return info


    def get_unit_objects(self):
        """ Return unit objects for the parsed input and desired units strings.
        """
        # fixme: Much error checking???
        # fixme: What does this do if the units aren't compatible?

        # if the units are "unknown", use the desired name to get the correct
        # units, otherwise the result will be dimensionless
        if self.desired_units.lower() == 'unknown':
            family_name = unit_manager.get_family_name(self.desired_name.lower())
            desired_units = unit_manager.get_unit_system().units(family_name)
            return desired_units, desired_units

        # Do unit conversion on each of the inputs.
        try:
            input_units = unit_parser.parse_unit(self.input_units,
                                                 suppress_unknown=False)
        except UnableToParseUnits:
            input_units = None

        try:
            desired_units = unit_parser.parse_unit(self.desired_units,
                                                   suppress_unknown=False)
        except UnableToParseUnits:
            desired_units = None

        return input_units, desired_units


    ##########################################################################
    # Protected interface.
    ##########################################################################

    #### Property get/set methods ############################################

    def _get_input_units_valid(self):
        """ Return whether the input_units string is a valid unit.
        """

        units = unit_parser.parse_unit(self.input_units)

        return units.is_valid()

    def _get_desired_units_valid(self):
        """ Return whether the desired_units string is a valid unit.
        """

        units = unit_parser.parse_unit(self.desired_units)

        return units.is_valid()

    def _get_units_compatible(self):
        """ Test whether las and desired units are valid and compatible.
        """

        input_units_obj = unit_parser.parse_unit(self.input_units)
        desired_units_obj = unit_parser.parse_unit(self.desired_units)

        if desired_units_obj.is_valid() and input_units_obj.is_valid():
            try:
                convert(1.0, input_units_obj, desired_units_obj)
                compatible = True
            except InvalidConversion:
                compatible = False
        else:
            compatible = False

        return compatible

    #### Static trait listeners ##############################################

    def _input_name_changed(self, value):
        """ Get the family name from the input name"""
        self.unit_family = self.unit_family.strip()

        if self.unit_family.lower() == 'unknown' or self.unit_family == '':
            self.unit_family = \
                unit_manager.get_family_name(self.input_name.lower())
        return

    def _desired_name_changed(self, value):
        """ Get the family name if it couldn't be retrieved from input name"""
        self.unit_family = self.unit_family.strip()

        if self.unit_family.lower() == 'unknown' or self.unit_family == '':
            self.unit_family = \
                unit_manager.get_family_name(self.desired_name.lower())
        return

    def _input_units_changed(self, value):
        """ Verify that the new unit systems are compatible. """

        self._update_units_status()

        return

    def _desired_units_changed(self, value):
        """ Verify that the new unit systems are compatible. """

        self._update_units_status()

        return

    #### Utility methods #####################################################

    def _update_units_status(self):
        """ Checks that units are valid and compatible.

            Sets unit_status appropriately.
        """

        if self.input_units_valid and self.desired_units_valid:
            if self.units_compatible:
                # all is well. units valid and compatible.
                self.units_status = "normal"
            else:
                # issue a warning about units not being compatible.
                self.units_status = "warning"
        else:
            # if either units are invalid, we have an error.
            self.units_status = "error"

        return



#### EOF ######################################################################
