""" Class for holding information about the "header" data in a file that hold
    log data.
"""
# Enthought library imports
from enthought.traits.api import HasTraits, List, Str, Float

# Geo library imports
from geo.io.log_conversion_info_item import LogConversionInfoItem
from geo.io.parameter_info_item import ParameterInfoItem


class LogFileHeaderBase(HasTraits):
    """ Abstract Base class handy for storing information about logs in a file.
    """

    #### 'LogFileHeaderBase' traits ############################################

    # Name of the well.
    well_name = Str

    # Floating point value used to indicate Nulls in data source.
    null_value = Float(-999.25)

    # List of ParameterInfoItem objects describing the logs in a data source.
    log_info = List(ParameterInfoItem)


    ############################################################################
    # 'LogFileHeaderBase' interface.
    ############################################################################

    @classmethod
    def from_file(cls, file_data):
        """ Read header from file_data and process it.

            file_data is a file-like object that is the source of the header.
            If you want to use a string, wrap it in StringIO.  The header must
            be at the beginning of the data.  Other data can follow the header
            and will be ignored.
        """
        new = cls()
        records = new._load_header(file_data)
        new._process_header(records)

        return new


    def log_conversion_info(self):
        """ Return a LogConversionInfoItem populated from information in this
            header.
        """
        info = [item.as_log_conversion_info_item() for item in self.log_info]

        # This 'repairs' the list renaming duplicate names, etc.
        info = LogConversionInfoItem.cleanup_list(info)

        return info


    ############################################################################
    # Protected interface.
    ############################################################################

    @staticmethod
    def _load_header(file):
        """ Return a list of "record" objects from the header file.

            These record objects can be anything you want -- they are processed
            by your _process_records method.
        """

        raise NotImplementedError


    def _process_records(self, records):
       """ Process the list of 'record' items and populate your object with the
           information in them.
       """

       raise NotImplementedError


#### EOF ######################################################################

