""" Read in a data file and create a new NumericContext.
    File formats supported : LAS, Tab-delimited,

    fixme: Better IO error reporting.
"""

# Standard imports
import string

# Enthought library imports.
from enthought.numerical_modeling.units.unit_array import UnitArray
from enthought.traits.api import Type, Str
from enthought.units.smart_unit import SmartUnit

# Geo library imports
from geo.data.api import LogContext
from geo.io.i_log_reader import ILogReader
from geo.io.log_file_header_to_log_reader_info_adapter import \
        LogFileHeaderToLogReaderInfoAdapter
from geo.io.util import as_file_object, read_data

# Global translator to ensure that log-context names are data-context-
# hierarchy friendly
trans_table = string.maketrans('/.\\-', '____')

class LogFileReader(ILogReader):
    """ Read in the data from a Las File based on the LasHeader
    """

    ############################################################################
    # 'ILogReader' interface.
    ############################################################################

    ### Public traits ##########################################################

    # Fileter that this reader can handle.
    file_filter = ''

    # header class used to generate a LogFileHeaderBase object for the file.
    header_class = Type

    # Bool to indicate if the file contains string data
    # This should be 'True' to read in string values for files which have a
    # column of string values
    has_string_data = False

    # Delimiter used in file formatting
    delimiter = Str('')

    ### Public methods #########################################################

    def read_info(self, source):
        """ Return a an object with ILogReaderInfo interface.

            'source' is either a file path or a file object.  If it is a file
            object, the 'source' traits in the returned  object
            is set to ''.
        """

        file = as_file_object(source)

        header = self.header_class.from_file(file)

        if not isinstance(source, basestring):
            source = ''

        return LogFileHeaderToLogReaderInfoAdapter(header=header,
                                                   source=source)


    def read_data(self, source, conversion_info):
        """ Return a numeric context from data file

            Parameters:
            -----------
            source: Str or File
                name of file or the file itself, which contains the data
            conversion_info: LogFileHeaderToLogReaderInfoAdapter
                contains information about the conversions between inputs
                and desired metadata

            Returns:
            --------
            context: Numeric Context
        """

        file = as_file_object(source)

        self.seek_to_data(file)

        log_names = [item.input_name for item in conversion_info.log_info]

        column_major = 1
        log_data = read_data(file, log_names, conversion_info.null_value,
                             self.has_string_data, column_major,
                             self.delimiter)
        log_collection = LogContext()

        for item in conversion_info.log_info:
            if item.load:
                # fixme: What to do if units aren't compatible??
                # Get the input and output unit objects (not strings)
                input_units, desired_units = item.get_unit_objects()

                # Get the data associated with this item.
                raw_data = log_data[item.input_name]

                # If this is the first log to be inserted into the log-context
                # check if it is a monotonically increasing log. If not, multiply
                # the values by -1. This should correct for logs which have the
                # index represented by elevation
                if len(log_collection.context_data) == 0 and\
                        len(raw_data) > 1 and raw_data[0] > raw_data[1]:
                    raw_data = raw_data * (-1)

                # Make a log from it and convert it to the right units.
                input_log = UnitArray(raw_data, units=input_units, copy=True)

                # If the desired_units is Nonetype then assign an invalid
                # SmartUnit as units for the desired_log, otherwise, convert
                # to desired_units if its different from input_units
                if desired_units is None:
                    desired_log = input_log
                    desired_log.units = SmartUnit(item.desired_units, 1,
                                                  (0,0,0,0,0,0,0), 0, False)
                else:
                    desired_log = input_log.as_units(desired_units)

                # Translate the name to have characters which are compliant with
                # context-names.
                final_name = str(item.desired_name).translate(trans_table)

                # Use copy to make sure we get a contiguous array.
                log_collection.set_dotted(final_name,desired_log)

        return log_collection

    # fixme : This method could move a level higher too.
    def seek_to_data(self, file):
        """ Find the appropriate location in the file to read data
        """
        raise NotImplementedError
