""" The SEGY Binary Reel Header.

The SEGY binary reel header contains meta information about the data. Much of
this information is optional, that is, the entire header is not required to be
valid. In fact, none of it is required to be valid, although some fields are
strongly recommended (as shown below by '*').

The 400 bytes contain 2-byte and 4-byte integers in the following layout::

    Bytes         Description
    -----         -----------
    001 - 004     Job identification number.
    005 - 008  *  Line number.
    009 - 012  *  Reel number.
    013 - 014  *  Number of data traces per record.
    015 - 016  *  Number of auxiliary traces per record.
    017 - 018  *  Sample interval of this reel's data in microseconds.
    019 - 020     Sample interval of original field recording in microseconds.
    021 - 022  *  Number of samples per trace for this reel's data.
    023 - 024     Number of samples per trace in original field recording.
    025 - 026  *  Data sample format code:
                      1  = 32-bit IBM floating point
                      2  = 32-bit fixed-point (integer)
                      3  = 16-bit fixed-point (integer)
                      4  = 32-bit fixed-point with gain code (integer)
                      >4 = 'host' floating-point format (eg. IEEE)
    027 - 028  *  CDP fold (expected number of data traces per ensemble).
    029 - 030     Trace sorting code:
                      1 = as recorded
                      2 = CDP ensemble
                      3 = single fold continuous profile
                      4 = horizontally stacked
    031 - 032     Vertical sum code (1 = no sum, 2 = two sum, ...)
    033 - 034     Sweep frequency at start in Hertz.
    035 - 036     Sweep frequency at end in Hertz.
    037 - 038     Sweep length in milliseconds.
    039 - 040     Sweep type code:
                      1 = linear
                      2 = parabolic
                      3 = exponential
                      4 = other
    041 - 042     Trace number of sweep channel.
    043 - 044     Sweep trace taper length at start in milliseconds.
    045 - 046     Sweep trace taper length at end in milliseconds.
    047 - 048     Taper type code:
                      1 = linear
                      2 = cosine squared
                      3 = other
    049 - 050     Correlated data traces (1 = no, 2 = yes).
    051 - 052     Binary gain recovered (1 = yes, 2 = no).
    053 - 054     Amplitude recovery method code:
                      1 = one
                      2 = spherical divergence
                      3 = AGC
                      4 = other
    055 - 056  *  Measurement system (1 = meters, 2 = feet).
    057 - 058     Impulse signal polarity (increase in pressure or upward
                  geophone case movement gives 1=negative or 2=positive number).
    059 - 060     Vibratory polarity code (seismic lags pilot signal by):
                      1 = 337.5 to 22.5 degrees
                      2 = 22.5 to 67.5 degrees
                      3 = 67.5 to 112.5 degrees
                      4 = 112.5 to 157.5 degrees
                      5 = 157.5 to 202.5 degrees
                      6 = 202.5 to 247.5 degrees
                      7 = 247.5 to 292.5 degrees
                      8 = 292.5 to 337.5 degrees
    061 - 400     Unassigned (for optional information).

"""

# Standard/built-in modules.
import struct

class BinaryHeader:
    """ A SEGY binary header. """

    # Format used by 'struct.unpack' to read the header.
    FORMAT = 'I' * 3 + 'H' * 24 + '340s'

    FIELDS = ['job',
          'line',
          'reel',
          'dataTracesPerRecord',
          'auxiliaryTracesPerRecord',
          'sampleInterval',
          'sampleIntervalOriginal',
          'samples',
          'samplesOriginal',
          'dataSampleFormat',
          'cdpFold',
          'traceSorting',
          'verticalSum',
          'sweepFrequencyStart',
          'sweepFrequencyEnd',
          'sweepLength',
          'sweepType',
          'sweepChannel',
          'sweepTaperLengthStart',
          'sweepTaperLengthEnd',
          'taperType',
          'correlatedDataTraces',
          'binaryGain',
          'amplitudeRecoveryMethod',
          'measurementSystem',
          'impulseSignalPolarity',
          'vibratoryPolarityCode',
          'unassigned'
          ]


    def __init__(self, data, byte_order):
        """ Constructor. """
        self.__unmarshal(data, byte_order)
        return

    ###########################################################################
    # Private interface.
    ###########################################################################

    def __unmarshal(self, data, byte_order):
        """ Unmarshal the binary header.

        'data' must be a string that is exactly 400 bytes long.

        """
        # Segy.LITTLE_ENDIAN = 10,   Segy.BIG_ENDIAN  = 20
        if byte_order == 20:
            c = ">"
        else:
            c = "<"

        # Unpack the data.
        values = struct.unpack(c + BinaryHeader.FORMAT, data)

        # Add each field as a public attribute.
        for i in range(len(values)):
            self.__dict__[BinaryHeader.FIELDS[i]] = values[i]

        return
