# PySegy modules.
from geo.io.segy.card_image_header import CardImageHeader
from geo.io.segy.binary_header import BinaryHeader
from geo.io.segy.trace import Trace

class Segy:
    """ A SEGY file.

    The SEGY data format consists of 3 parts:-

    The first part is a 3200 byte EBCDIC card image header which contains 40
    cards worth of text (i.e. 40 lines of text with 80 characters per line)
    describing the tape.

    The second part is a 400 byte binary header containing information about
    the contents of the tape reel.

    The third portion of the SEGY format consists of the actual seismic
    traces. Each trace has a 240 byte trace header. The data follow, written
    in one of 4 possible 32-bit formats. Currently we support IBM and IEEE
    floating point formats.

    """

    CARD_IMAGE_HEADER_LEN = 3200
    BINARY_HEADER_LEN     = 400
    TRACE_HEADER_LEN      = 240

    # The data format of the trace samples.
    IBM  = 1
    IEEE = 5

    # Byte-order.
    LITTLE_ENDIAN = 10
    BIG_ENDIAN  = 20


    def __init__(self, filename, data_format=None, byte_order=None):
        """ Constructor.

        The default data format is IBM floating point.
        The default byte-order is BIG endian.

        """
        print "**** Creating SEGY for", filename
        self.__filename = filename

        # By default, the data is in IBM floating point format. However in true
        # SEGY tradition, we allow the user to tell us what is *really* in
        # there!
        #
            # Currently, we support only IBM and IEEE floating point formats.
        self.__data_format = data_format or Segy.IBM

        # By default, the byte-order is BIG endian.
        #
        # fixme: Should default to native byte order.
        self.__byte_order = byte_order or Segy.BIG_ENDIAN

        # Read the entire file, creating an object model of the SEGY data as
        # we go.
        self.__read_file(filename)

        # List constructor.
        #list.__init__(self, self.__traces)

        return

    ###########################################################################
    # 'Object' interface.
    ###########################################################################

    def __repr__(self):
        return "SEGY(" + self.__filename + ")"

    ###########################################################################
    # 'List' interface.
    ###########################################################################
    def __len__(self):
        return len(self.__traces)

    def __getitem__(self, idx):
        return self.__traces[idx]

    def __setitem__(self, idx, value):
        self.__traces[idx] = value

    ###########################################################################
    # 'Segy' interface.
    ###########################################################################

    def get_filename(self):
        """ Returns the name of the SEGY file. """

        return self.__filename

    filename = property(get_filename, None, None, None)

    def get_card_image_header(self):
        """ Returns the card image header.

        The card image header is 3200 bytes of EBCDIC that is rarely used!

        """

        return self.__card_image_header

    card_image_header = property(get_card_image_header, None, None, None)

    def get_binary_header(self):
        """ Returns the binary header. """

        return self.__binary_header

    binary_header = property(get_binary_header, None, None, None)

    def get_traces(self):
        """ Returns the actual traces. """

        return self.__traces

    traces = property(get_traces, None, None, None)

    ###########################################################################
    # Private interface.
    ###########################################################################

    def __read_file(self, filename):
        """ Read the entire SEGY file. """

        # Open the file in BINARY mode.
        f = file(filename, 'rb')

        # Card image header.
        data = f.read(Segy.CARD_IMAGE_HEADER_LEN)
        self.__card_image_header = CardImageHeader(data, self.__byte_order)

        # Binary header.
        data = f.read(Segy.BINARY_HEADER_LEN)
        self.__binary_header = BinaryHeader(data, self.__byte_order)

        # Traces.
        self.__traces = self.__read_traces(f, self.__binary_header.samples)

        # All done!
        f.close()

        return

    def __read_traces(self, f, samples):
        """ Read the trace data from the file. """

        traces = []

        sample_size = samples * 4

        trace_size = Segy.TRACE_HEADER_LEN + (samples * 4)

        header_data = f.read(Segy.TRACE_HEADER_LEN)
        while header_data != "":
            sample_data = f.read(sample_size)

            traces.append(Trace(header_data, sample_data, self.__data_format,
                                self.__byte_order))

            header_data = f.read(Segy.TRACE_HEADER_LEN)

        return traces


