# Standard/built-in modules.
import struct
import numpy

# Local imports
from trace_header import TraceHeader



class Trace:
    """ A single trace in a SEGY file.

    A trace consists of a trace header and a list of the actual trace samples.

    """

    def __init__(self, header_data, sample_data, data_format, byte_order):
        """ Constructor. """

        self.__data_format = data_format
        self.__byte_order = byte_order

        # We unmarshal the trace header immediately...
        self.__header = TraceHeader(header_data, byte_order)

        # ...and the samples lazily!
        self.__samples = self.__unmarshal_samples(sample_data, data_format,
                                                  byte_order)

        # List constructor.
        #list.__init__(self, self.__samples)

        return

    ###########################################################################
    # 'Object' interface.
    ###########################################################################
    def __repr__(self):
        return "Trace(" + `self.__header.lineSequence` + ")"

    ###########################################################################
    # 'List' interface.
    ###########################################################################
    def __len__(self):
        return len(self.__samples)

    def __getitem__(self, idx):
        return self.__samples[idx]

    def __setitem__(self, idx, value):
        self.__samples[idx] = value

    ###########################################################################
    # 'Trace' interface.
    ###########################################################################

    def get_header(self):
        """ Returns the trace header. """

        return self.__header

    header = property(get_header, None, None, None)

    def get_samples(self):
        """ Returns the trace samples. """

        return self.__samples

    samples = property(get_samples, None, None, None)

    ###########################################################################
    # 'Private' interface.
    ###########################################################################

    def __unmarshal_samples(self, data, data_format, byte_order):

        # Segy.BIG_ENDIAN = 20, Segy.LITTLE_ENDIAN = 10
        if byte_order == 20:
            c = ">"
        else:
            c = "<"


        # Segy.IBM  = 1,  Segy.IEEE = 5
        if data_format == 1:
            samples = struct.unpack(c + "i"*self.__header.samplesInTrace, data)
            samples = numpy.array(samples)
            samples = self.ibm2ieee(samples)
        else:
            samples = struct.unpack(c + "f"*self.__header.samplesInTrace, data)
            samples = numpy.array(samples)

        return samples

    def ibm2ieee(self, ibm):
        """ Converts an IBM floating point number into IEEE format. """

        sign = ibm >> 31 & 0x01

        exponent = ibm >> 24 & 0x7f

        mantissa = ibm & 0x00ffffff
        mantissa = (mantissa * 1.0) / pow(2, 24)

        ieee = (1 - 2 * sign) * mantissa * pow(16.0, exponent - 64)

        return ieee
