""" The SEGY Trace Header.


The 240-byte binary trace header consists of 2-byte and 4-byte integers in the
following layout::

    Bytes         Description
    -----         -----------
    001 - 004  *  Trace sequence number within line.
    005 - 008     Trace sequence number within reel.
    009 - 012  *  Original field record number.
    013 - 016  *  Trace sequence number within original field record.
    017 - 020     Energy source point number.
    021 - 024     CDP ensemble number.
    025 - 028     Trace sequence number within CDP ensemble.
    029 - 030  *  Trace identification code:
                      1 = seismic data
                      2 = dead
                      3 = dummy
                      4 = time break
                      5 = uphole
                      6 = sweep
                      7 = timing
                      8 = water break
                      9+ = optional use
    031 - 032     Number of vertically summed traces yielding this trace.
    033 - 034     Number of horizontally stacked traced yielding this trace.
    035 - 036     Data use (1 = production, 2 = test).
    037 - 040     Distance from source point to receiver group.
    041 - 044     Receiver group elevation.
    045 - 048     Surface elevation at source.
    049 - 052     Source depth below surface.
    053 - 056     Datum elevation at receiver group.
    057 - 060     Datum elevation at source.
    061 - 064     Water depth at source.
    065 - 068     Water depth at receiver group.
    069 - 070     Scalar for elevations and depths (+ = multiplier,- = divisor).
    071 - 072     Scalar for coordinates (+ = multiplier, - = divisor).
    073 - 076     X source coordinate.
    077 - 080     Y source coordinate.
    081 - 084     X receiver group coordinate.
    085 - 088     Y receiver group coordinate.
    089 - 090     Coordinate units(1= length in meters or feet, 2= arc seconds).
    091 - 092     Weathering velocity.
    093 - 094     Subweathering velocity.
    095 - 096     Uphole time at source.
    097 - 098     Uphole time at receiver group.
    099 - 100     Source static correction.
    101 - 102     Receiver group static correction.
    103 - 104     Total static applied.
    105 - 106     Lag time between end of header and time break in milliseconds.
    107 - 108     Lag time between time break and shot in milliseconds.
    109 - 110     Lag time beteen shot and recording start in milliseconds.
    111 - 112     Start of mute time.
    113 - 114     End of mute time.
    115 - 116  *  Number of samples in this trace.
    117 - 118  *  Sample interval of this trace in microseconds.
    119 - 120     Field instrument gain type code:
                      1 = fixed
                      2 = binary
                      3 = floating point
                      4+ = optional use
    121 - 122     Instrument gain constant.
    123 - 124     Intrument early gain in decibels.
    125 - 126     Correlated (1 = no, 2 = yes).
    127 - 128     Sweep frequency at start.
    129 - 130     Sweep fequency at end.
    131 - 132     Sweep length in milliseconds.
    133 - 134     Sweep type code:
                      1 = linear
                      2 = parabolic
                      3 = exponential
                      4 = other
    135 - 136     Sweep taper trace length at start in milliseconds.
    137 - 138     Sweep taper trace length at end in milliseconds.
    139 - 140     Taper type code:
                      1 = linear
                      2 = cosine squared
                      3 = other
    141 - 142     Alias filter frequency.
    143 - 144     Alias filter slope.
    145 - 146     Notch filter frequency.
    147 - 148     Notch filter slope.
    149 - 150     Low cut frequency.
    151 - 152     High cut frequency.
    153 - 154     Low cut slope.
    155 - 156     High cut slope.
    157 - 158     Year data recorded.
    159 - 160     Day of year.
    161 - 162     Hour of day (24-hour clock).
    163 - 164     Minute of hour.
    165 - 166     Second of minute.
    167 - 168     Time basis (1 = local, 2 = GMT, 3 = other).
    169 - 170     Trace weighting factor for fixed-point format data.
    171 - 172     Geophone group number of roll switch position one.
    173 - 174     Geophone group number of first trace of original field record.
    175 - 176     Geophone group number of last trace of original field record.
    177 - 178     Gap size (total number of groups dropped).
    179 - 180     Overtravel associated with taper(1= down/behind, 2= up/ahead).
    181 - 240     Unassigned (for optional information).



"""

# Standard/built-in modules.
import struct

class TraceHeader:
    """ A SEGY trace header. """

    FORMAT = 'I' * 7 + 'H' * 4 + 'I' * 8 + 'H' * 2 + 'I' * 4 + 'H' * 46 + \
             'I' * 5 + 'H' * 2 +'36s'

    FIELDS = ['lineSequence',
          'reelSequence',
          'fieldOriginal',
          'traceSequence',
          'energySourcePoint',
          'cdpEnsemble',
          'traceSequence',
          'traceId',
          'verticallySummedTraces',
          'horizontallyStackedTraces',
          'dataUse',
          'distanceFromSource',
          'receiverElevation',
          'surfaceElevation',
          'sourceDepth',
          'datumElevationAtReceiver',
          'datumElevationAtSource',
          'waterDepthAtSource',
          'waterDepthAtReceiver',
          'scalarElevations',
          'scalarCoordinates',
          'xSource',
          'ySource',
          'xReceiver',
          'yReceiver',
          'coordinateUnits',
          'weatheringVelocity',
          'subweatheringVelocity',
          'upholeTimeAtSource',
          'upholeTimeAtReceiver',
          'sourceStaticCorrection',
          'receiverStaticCorrection',
          'totalStatic',
          'lagTimeHeaderAndBReak',
          'lagTimeBreakAndShot',
          'lagTimeShotAndRecording',
          'startMuteTime',
          'endMuteTime',
          'samplesInTrace',
          'sampleInterval',
          'fieldGainType',
          'instrumentGainConstant',
          'instrumentEarlyGain',
          'correlated',
          'sweepFrequencyStart',
          'sweepFrequencyEnd',
          'sweepLength',
          'sweepType',
          'sweepTraceLengthStart',
          'sweepTraceLengthEnd',
          'taperType',
          'aliasFilterFrequency',
          'aliasFilterSlope',
          'notchFilterFrequency',
          'notchFilterSlope',
          'lowCutFrequency',
          'highCutFrequency',
          'lowCutSlope',
          'highCutSlope',
          'year',
          'day',
          'hour',
          'minute',
          'second',
          'timeBasis',
          'weightingFactor',
          'geophoneRoll',
          'geophoneFirstTrace',
          'geophoneLastTrace',
          'gapSize',
          'overtravel',
          'xCoordCDP',
          'yCoordCDP',
          'inLineNumber',
          'crossLineNumber',
          'shotPointNumber',
          'scalarForShotPoint',
          'traceUnits',
          'unassigned'
          ]

    def __init__(self, data, byte_order):
        """ Constructor. """

        self.__unmarshal(data, byte_order)
        self.raw_data = data

        return

    ###########################################################################
    # Private interface.
    ###########################################################################

    def __unmarshal(self, data, byte_order):
        """ Unmarshal the trace header.

        'data' must be a string that is exactly 240 bytes long.

        """

        # Segy.BIG_ENDIAN = 20 ; Segy.LITTLE_ENDIAN = 10
        if byte_order == 20:
            c = ">"
        else:
            c = "<"

        # Unpack the data.
        values = struct.unpack(c + TraceHeader.FORMAT, data)

        # Add each field as a public attribute.
        for i in range(len(values)):
            self.__dict__[TraceHeader.FIELDS[i]] = values[i]

        return
