#-----------------------------------------------------------------------------
#
#  Copyright (c) 2007 by Enthought, Inc.
#  All rights reserved.
#
#-----------------------------------------------------------------------------

""" UI support for editing Tops
"""

# ETS library imports
from enthought.traits.api import HasTraits, List, Any, Instance
from enthought.traits.ui.api import Handler, TableEditor, View, Item
from enthought.traits.ui.table_column import ObjectColumn
from enthought.numerical_modeling.units.unit_array import UnitArray
from enthought.units.api import dimensionless

# Geo library imports
from geo.io.tops.tops_edit_item import TopsEditItem
from geo.data.tops_context import TopsContext

# Table editor definition
table_editor = TableEditor(columns = [ObjectColumn(name='depth',label='Depth'),
                                      ObjectColumn(name='value',label='Value')],
                           deletable=True,
                           row_factory=TopsEditItem,
                           filters = [],
                           sortable=False,
                           configurable=False)


#-------------------------------------------------------------------------------
#   TopsEditUIHandler
#-------------------------------------------------------------------------------

class TopsEditUIHandler(Handler):
    """ Handler for TopsEditUI
    """

    edit_ui = Instance('TopsEditUI')

    def closed(self, info, is_ok):
        """ Postprocessing the editing of Tops
        """

        if is_ok:
            # Return the edited tops-context
            self.edit_ui.edit_tops_context()
            return

#-------------------------------------------------------------------------------
#   TopsEditUI
#-------------------------------------------------------------------------------

class TopsEditUI(HasTraits):
    """ Class for UI support for Tops
    """

    tests = List(TopsEditItem)

    depth_name = ''
    tops_name = ''

    tops_context = Instance('geo.data.tops_context.TopsContext')

    edit_view = View(Item('tests', editor=table_editor, show_label=False),
                     title='Edit Tops',
                     height=300, resizable=True,
                     buttons=['OK', 'Cancel', 'Help'],
                     apply=False, revert=False, undo=False, help=False)


    ### private methods --------------------------------------------------------

    def _tops_context_changed(self, trait, old, new):
        """ Changing Tops Context should populate the tests
        """

        depth_log = []
        tops_log = []

        self.tests = []
        self.depth_name = ''
        self.tops_name = ''

        for k,v in self.tops_context.items():
            if isinstance(v, UnitArray) and v.units is not None:
                if v.units.derivation == (1,0,0,0,0,0,0):
                    self.depth_name, depth_log = k,v
                else:
                    self.tops_name, tops_log = k,v
            else:
                self.tops_name, tops_log = k,v

        if len(depth_log) == len(tops_log) and len(depth_log) != 0:
            for i in range(len(depth_log)):
                self.tests.append(TopsEditItem(depth = depth_log[i],
                                              value = tops_log[i]))

        return

    ### public methods ---------------------------------------------------------

    def edit_tops_context(self):
        """ Edit the Topscontext
        """

        depth_log = []
        tops_log = []
        depth_units = None

        for t in self.tests:
            depth_log.append(t.depth)
            tops_log.append(t.value)

        # Retrieving units
        init_depth_log = self.tops_context.get_dotted(self.depth_name)
        if hasattr(init_depth_log, 'units'):
            depth_units = init_depth_log.units

        init_tops_log = self.tops_context.get_dotted(self.tops_name)

        # Removing old logs
        self.tops_context.pop(self.depth_name)
        self.tops_context.pop(self.tops_name)

        # Replacing with new logs
        self.tops_context.set_dotted(self.depth_name,
                                     UnitArray(depth_log,
                                               units=depth_units))
        self.tops_context.set_dotted(self.tops_name,
                                     UnitArray(tops_log,
                                               units=dimensionless))

        return


### EOF ------------------------------------------------------------------------
