# Standard library imports
from fnmatch import fnmatch
from copy import deepcopy
from string import maketrans, translate
import os.path

# Enthought library imports
from enthought.traits.api import (HasTraits, List, Instance, File, Bool,
                                  Str)
from enthought.traits.ui.api import View, Item, HGroup, Tabbed, Group
from enthought.traits.ui.editors import FileEditor
from enthought.traits.ui.api import CodeEditor

# Geo library imports
from geo.io.csv_log.csv_log_reader import CSVLogReader
from geo.io.i_log_reader import ILogReader
from geo.io.i_log_reader_info import ILogReaderInfo
from geo.io.las.api import LasLogReader
from geo.io.log_conversion_info_item import LogConversionInfoItem
from geo.io.whitespace_log.whitespace_log_reader import WhitespaceLogReader

class TextView(HasTraits):
    """ Helper class for viewing contents of text file.
    """

    data = Str

    view = View(Item(name = 'data',
                     editor = CodeEditor(),
                     style = 'readonly',
                     show_label = False,
                     resizable = True,
                     height = 400,
                     width = 650
                ),
                resizable = True,
           )


class FileLogReaderUI(HasTraits):
    """ UI for reading logs from file-based data sources such as las, txt, etc.

        The UI supports multiple different types of file readers.  The list
        of available readers is configurable and stored in the 'readers'
        variable.
    """

    # File name to read
    file_name = File

    # The reader currently used to read file_name.
    active_reader = Instance(ILogReader)

    # List of readers that are available for reading files.
    readers = List([LasLogReader(), CSVLogReader(), WhitespaceLogReader()],
                   ILogReader)

    # Contains information about the logs available in the file.
    log_reader_info = Instance(ILogReaderInfo)

    # Select or deselect all logs 'in mas'
    select_deselect_all = Bool(True)

    # Toggle whether output names are all lower case.
    lower_case = Bool(False)

    # A text view of the file.
    text_view = Instance(TextView)


    ############################################################################
    # HasTraits interface
    ############################################################################

    def trait_view(self, name=None, view_element=None):
        """ We want to set the file filters dinamically, so we'll build the
            view on-the-fly instead of specifying a static one.

        """

        # Including all the filters accomodated by the reader
        filters = []
        for item in self.readers:
            if isinstance(item.file_filter, list):
                filters.extend(item.file_filter)
            else:
                filters.append(item.file_filter)

        view = View(Item('file_name', editor=FileEditor(filter=filters, entries=0),
                         label="File"),
                    '_',
                    Tabbed(
                           Group(
                                 Group(
                                       # fixme: Wish we didn't have to specify
                                       #        height here.
                                       Item('log_reader_info',show_label=False,
                                       style='custom',height=375),
                                 ),
                                 '_',
                                 HGroup(Item('select_deselect_all',
                                             label='Select/deselect all'
                                        ),
                                        '_',
                                        Item('lower_case'),
                                 ),
                                 label='Select Curves',
                           ),
                           Group(Item('text_view',
                                      style='custom',
                                      show_label=False
                                 ),
                                 label='File View',
                           ),
                    ),
                    buttons=["OK", "Cancel", "Help"],
                    help_id="geolib|HID_Open_Log_File_dlg",
                    title="Open Log File",
                    width=700,
                    height=600,
                    resizable=True)
        return view


    ############################################################################
    # FileLogReaderUI interface
    ############################################################################

    ### Trait change listeners #################################################

    def _file_name_changed(self, old, new):
        """ When the file changes, we need to update the log_reader_info.

            fixme: Need error detection on IO errors here.
        """
        for reader in self.readers:

            # Find the reader that has a matching file filter, and use it
            # to read the header info.
            try:
                # Check if the file filter is a list and check with each filter
                # in the list
                # Truncate the extension of each filter and match with the
                # extension of filename
                if isinstance(reader.file_filter, list):
                    truth_value = False
                    for filter in reader.file_filter:
                        truncated_file_ext = filter[filter.rfind('|')+1:]
                        truth_value = truth_value or \
                            fnmatch(new, truncated_file_ext)
                else:
                    truncated_file_ext = \
                        reader.file_filter[reader.file_filter.rfind('|')+1:]
                    truth_value = fnmatch(new, truncated_file_ext)

                if truth_value:
                    self.active_reader = reader
                    self.log_reader_info = self.active_reader.read_info(new)

                    # Setting the well name if it is empty
                    if self.log_reader_info.well_name == '' or\
                        self.log_reader_info.well_name is None:
                        
                        self.log_reader_info.well_name =\
                            os.path.splitext(os.path.basename(self.file_name))[0]

                    if self.log_reader_info.well_name == '' or\
                        self.log_reader_info.well_name is None:
                        self.log_reader_info.well_name = 'context'

                    break

            except:
                pass
        else:
            # If we fail, perhaps attempt with a "default reader?"
            #error("File type not recognized.")
            self.active_reader = None
            self.log_reader_info = None

        self._update_text_view()

    def _select_deselect_all_changed(self, old, new):
        """ Toggle all logs to the selected or unselected state.

            fixme: tortoise svn has a slightly more sophisticated
            select/deselect mechanism that we may want to mimic
        """
        if self.log_reader_info is not None:
            for item in self.log_reader_info.log_info:
                item.load = new

    def _lower_case_changed(self, old, new):
        """ Toggle all desired names to lower case or to use default.

            fixme: This algorithm doesn't quite work correctly if their are
                   duplicate names running around in the file.
                   Try same_mneumonic.las.
        """
        if self.log_reader_info is not None:
            # get a copy of the log information, and modify it if necessary.
            log_info = deepcopy(self.log_reader_info.log_info)
            for item in log_info:
                if new is True:
                    # Force all names to lower case.
                    item.desired_name = item.desired_name.lower()
                else:
                    if item.desired_name.lower() == item.input_name.lower():
                        # If the names are the same, then it was likely changed
                        # by a previous lower casing process.
                        item.desired_name = item.input_name
                    else:
                        # The user has edited the variable, lets don't mess
                        # with it.
                        pass

            # The lower casing process could have made duplicate names
            # so we clean them up before writing them back to the actual list.
            cleaned_info = LogConversionInfoItem.cleanup_list(log_info)
            self.log_reader_info.log_info = cleaned_info

    def _update_text_view(self):
        """ Load the specified file into the text viewer.
        """
        las_text = '\nCould not open or read LAS file: %s' % self.file_name
        las_text += '\n\nClick Browse to select an LAS file.\n'
        try:
            las_text = open(self.file_name).read()
        except Exception, msg:
            las_text += '\n %s ' % msg

        self.text_view = TextView(data = las_text)
        return self.text_view



if __name__ == '__main__':
    file_name = r"C:\wrk\cp\src\lib\cp\io\tests\sample.las"
    reader = FileLogReaderUI(file_name=file_name)
    reader.configure_traits()
