#-------------------------------------------------------------------------------
#
#  Define the Log information database access class
#
#  Written by: David C. Morrill
#
#  Date: 01/18/2004
#
#  (c) Copyright 2004 by Enthought, Inc.
#
#  Classes defined: LogInfo
#
#  Singleton objects defined: log_info
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  Imports:
#-------------------------------------------------------------------------------

import csv
import os

from enthought.units.unit_manager import unit_manager
from enthought.util.resource import get_path
from enthought.logger import logger
from enthought.logger.log_point import log_point

from enthought.units.unit_parser    import unit_parser

#-------------------------------------------------------------------------------
#  'LogInfo' class:
#-------------------------------------------------------------------------------

class LogInfo:
    """ Creates a database of commonly accessed Log information, such as 
        description, default scale, line color, line style, units and
        plot scale ranges for different unit systems.
    """
    
    _unit_system_columns = [ 'UNITS', 'LEFT', 'RIGHT' ]

    #---------------------------------------------------------------------------
    #  Initialize the object:
    #---------------------------------------------------------------------------
    
    def __init__ ( self, aliases ):
        
        self.aliases      = aliases
        self.log_names    = log_names = {}
        self.column_names = column_names = {}
        self.unit_systems = unit_systems = []
        
        is_data           = False
        fh                = file( os.path.join( get_path( self ), 
                                                'data', 'log_info.txt' ) )
        
        for data in csv.reader( fh, delimiter = ' ', skipinitialspace = True ):
            if (len( data ) == 0) or (data[0][0:1] == '#'):
                # Ignore blank lines and comment lines:
                pass
            elif is_data:
                log_names[ data[0].lower() ] = map( lambda func, x: func( x ),
                                                    converters, data )
            else:
                # Parse the header line 
                is_data      = True
                column_names = self.column_names
                converters   = []
                for i, column in enumerate( data ):
                    column_names[ column ] = i
                    if (column[-5:] == '_LEFT') or (column[-6:] == '_RIGHT'):
                        converters.append( cvt_float )
                    elif column[-6:] == '_UNITS':
                        converters.append( cvt_unit )
                        unit_systems.append(column[:-6])
                    else:
                        converters.append( cvt_str )
        fh.close()
        
    #---------------------------------------------------------------------------
    #  Return current unit system eg 'kgs','metric','imperial'
    #---------------------------------------------------------------------------
    def get_unit_system(self):
        unitsys = unit_manager.get_default()
        return unitsys
    

    #---------------------------------------------------------------------------
    #  Return the family name corresponding to a specified log name:
    #---------------------------------------------------------------------------
    
    def get_family_name ( self, log_name ):
        return self.aliases.get_family_name( log_name ) 
        
    def get_inverse_family_name(self, log_name):
        family = self.aliases.get_family_name( log_name ) 
        inverse = self(family,column_name='INVERSE')
        if inverse == "none":
            inverse = ''
        return inverse

    def get_inverse_log_name(self, log_name):
        """ Returns the "default" name to use for inverted logs. 
        
            This assumes that the first name in the alias list is
            the "primary" family name,  i.e., the one to use in 
            ProAVA2.
        """
        family = self.get_inverse_family_name(log_name)
        return self.aliases.preferred_names.get(family, '')
            
    #---------------------------------------------------------------------------
    #  Return a value for a specified log:
    #---------------------------------------------------------------------------
    
    def __call__ ( self, family_name, column_name = 'SCALE' ):
        """
        This currently returns the data for the active/default unit system
        """
        column_name = column_name.upper()
        if column_name in self._unit_system_columns:
            column_name = '%s_%s' % (self.get_unit_system(), column_name )
        
        try:
            return self.log_names[ family_name ][ self.column_names[ column_name ] ]
        except:
            print 'No log attribute found for %s %s' % (family_name, column_name)
            return None
    
#-------------------------------------------------------------------------------
#  Convert a string to itself or None if its values is '?':
#-------------------------------------------------------------------------------

def cvt_str ( text ):
    if text == '?':
        return None
    return text
    
#-------------------------------------------------------------------------------
#  Convert a string to its floating point value or None if its value is '?':
#-------------------------------------------------------------------------------
    
def cvt_float ( text ):
    if text == '?':
        return None
    return float( text )
    
#-------------------------------------------------------------------------------
#  Parse a unit description
#-------------------------------------------------------------------------------
    
def cvt_unit ( unit_label ):
    units = unit_parser.parse_unit(unit_label)
    return units      
    

class LogNameAliases:
    
    def __init__(self):
        self.names = {}
        self.preferred_names = {}
        
        ##print 'Reading the log_aliases file'
        
        fh = file( os.path.join( get_path( self ), 'data', 'log_aliases.txt' ) )
        for data in csv.reader( fh, delimiter = ' ', skipinitialspace = True ):
            # Ignore blank lines and comment lines:
            if (len( data ) == 0) or (data[0][0:1] == '#') or data[0].startswith("FAMILY"):
                pass
            else:
                family_name = data[0].lower()
                # add the family name to the alias list as well ....
                for alias in data:
                    if self.names.has_key(alias):
                        print 'Warning: duplicate key: %s in log_alias.txt' % alias
                    self.names[alias] = family_name 
                
                # set up the preferred log name for the family to be the first one
                # in the alias list if possible.  Otherwise just use the family_name
                if len(data) > 1:    
                    self.preferred_names[family_name] = data[1].lower()
                else:    
                    self.preferred_names[family_name] = data[1].lower()
        return
        
    def get_family_name(self, name):
        
        match = name.lower().split('_')
        while len(match) > 0:
            family = self.names.get('_'.join(match))
            if family is not None:
                break
                
            match = match[:-1]
        
        else:
            family = 'unknown'
            logger.warn('Could not find the family name for %s %s' % (name, log_point()) )
            #log_point('======================================================')

        return family
        
#-------------------------------------------------------------------------------
#  Create the singleton LogInfo object:
#-------------------------------------------------------------------------------
log_aliases = LogNameAliases()
log_info    = LogInfo( log_aliases )

