# Utility functions and classes for reading data.

# Numeric library imports
from numpy import array, NaN
import logging, string

# Enthought library imports
from enthought.units.unit_parser import unit_parser

# Global logger
logger = logging.getLogger(__name__)

#-------------------------------------------------------------------------------
# Exception Classes
#-------------------------------------------------------------------------------

class InvalidDataError(Exception):
    """ Exception when data is of incorrect format
    """
    def __str__(self):
        return "Either data is missing or is of invalid format in the file"


class InvalidHeaderError(Exception):
    """ Exception when header is of incorrect format
    """
    def __str__(self):
        return "Either names or units are of inappropriate format"


#-------------------------------------------------------------------------------
# Data retrieval methods
#-------------------------------------------------------------------------------
def as_file_object(source):
    """ Return a file object for source.

        Source can be either a file or a file path.  If it is a file, it
        is simply returned.  If it is a string, we attempt to open the
        string and return the resulting file object.
    """
    if isinstance(source, basestring):
        file = open(source)
    else:
        file = source

    return file


def find_duplicates(lst):
    """ Find the indices of duplicate items in a list


        Inputs:
        lst -- a list of items.  Elements in lst must be hasable.

        Outputs:
        result -- dictionary (item->list of indices)

        example:
        >>> from geo.io.util import find_duplicates
        >>> lst = ['a','b','a','b','a','c']
        >>> find_duplicates(lst)
        {'a': [0, 2, 4], 'b': [1, 3]}
    """

    # Fill a dictionary with key->indices list.
    dups = []
    indices = {}
    for i, item in enumerate(lst):
        if item in indices:
            indices[item].append(i)
            if item in dups:
                dups.remove(item)
        else:
            indices[item] = [i]
            dups.append(item)

    # now delete all entries that are length 1.
    for item in dups:
        del indices[item]

    return indices


def is_number(value):
    for f in [int, float]:
        try:
            f(value)
        except ValueError:
            continue
        return True

    return False


def load_header_for_delimited_files( file, delimiter = None, fill_units = True ):
        """ Return a list (name, unit) pairs for each of the logs in the file.

            'delimiter' specifies the splitting criterion.
            'fill_units' determines rigorous checking of units

        """

        log_names = []
        log_units = []
        log_names_found = False

        # loop until we read a line for the log units.
        for line in file:
            line = line.strip()
            if line.startswith('#') or line == '':
                # ignore lines that are comments or whitespace
                pass
            elif not log_names_found:
                # process first non-comment/blank line as whitespace-delimited
                # log names.
                if delimiter == '' or delimiter is None:
                    log_names = line.split()
                else:
                    log_names = line.split(delimiter)

                log_names_found = True
            else:
                # Final case is that this is the units line.
                # Once we've found this, it is time to stop.
                if delimiter == '' or delimiter is None:
                    log_units = line.split()
                else:
                    log_units = line.split(delimiter)

                if fill_units:
                    for i in range(len(log_units)):
                        log_units[i] = log_units[i].strip()
                        if log_units[i] == '' or log_units[i] == '""':
                            log_units[i] = 'None'
                break

        # Checking if we found the same number of names and units.
        # Check if we can supplement the remaining units with a smart choice
        if len(log_names) > len(log_units) and fill_units:
            unit_length = len(log_units)
            for i in range(unit_length,len(log_names)):
                log_units.append(unit_parser.parse_unit(log_names[i]).label)

        if len(log_names) != len(log_units):
            msg = "Length of name(%s) and unit(%s) entries in header" \
                  " do not match" % (len(log_names), len(log_units))
            raise AssertionError, msg

        # Checking if log_names and log_units do not contain floats,
        # thus checking validity of the names and units
        for name in log_names:
            if is_number(name):
                raise InvalidHeaderError
        for unit in log_units:
            if is_number(unit):
                raise InvalidHeaderError

        return zip(log_names, log_units)


def read_data(file, log_names, null_value, has_string_data = False,
              column_major = 1, delimiter = None):
    """ Read the data out of a File.
        It is assumed here that the file reader is set at the right position
    """

    #TODO: Write a more generic function to include handling files containing
    #      more than 2 columns, and some columns containing string data

    column_count = len(log_names)

    if has_string_data == False:
        data = _read_data(file, column_count, null_value, delimiter)
    elif column_count == 2:
        (data_float, data_string) = _read_two_columned_string_data(file,
                                                                   null_value)
        return {log_names[0]: data_float, log_names[1]: data_string}
    else:
        raise InvalidDataError

    # Put each log in rows instead of columns if data is in column-major
    # arrangement
    if column_major == 1 :
        data=data.transpose()
    else:
        pass

    return dict(zip(log_names, data))


def _read_two_columned_string_data(file, null_value):
    """ Simple routine to read a file containing a column of
        floats and column of strings as data

    """

    data_float = []
    data_string = []

    for line in file:
        line = line.strip()
        data = line.split()
        if len(data) == 0 :
            # Passing blanklines
            pass
        else:
            if null_value is not None and data[0] == null_value:
                data_float.append(NaN)
            else:
                data_float.append(float(data[0]))

            data.pop(0)
            data_string.append(' '.join(data))

            if null_value is not None and data_string[-1] == null_value:
                data_string[-1] = 'N/A'
            elif data_string[-1] == '':
                raise InvalidDataError

    if len(data_float) == 0 or len(data_string) == 0 or \
        len(data_float) != len(data_string):
        raise InvalidDataError

    return (array(data_float), array(data_string))


def _read_data(file, column_count, null_value=None, delimiter=None):
    """ Move file to the first line of the data section.
        fixme: Better error reporting.
    """
    # Read all of the remaining data in one go into a string
    # This may be a problem for really large data sets?
    try:
        # Check if there are any comment lines prior to reading data
        init_data = file.readline()
        while init_data[0] == '#' or init_data[0] == '\n' or init_data[0:2] == '\r\n':
            init_data = file.readline()

        if delimiter is None or delimiter == '':
            split_initdata = init_data.split()
        else:
            split_initdata = init_data.split(delimiter)

        # Read all the data from the file.
        data = file.read(-1)

        # split data on whitespace
        if delimiter is None or delimiter == '':
            splitdata = data.split()
        else:
            # split data on newline and commas
            trans = string.maketrans('\n', delimiter)
            newdata = data.translate(trans)
            # The last \n should be removed, if present.
            if newdata[-1] == delimiter:
                newdata = newdata[:-1]
            splitdata = newdata.split(delimiter)

        for i in range(0,splitdata.count('\r')):
            splitdata.remove('\r')

        # First we try a fast technique ...
        try:
            if len(split_initdata):
                vals = [float(x) for x in split_initdata]
                vals.extend([float(x) for x in splitdata])
            else:
                vals = [ float(x) for x in splitdata ]
        except Exception, msg:
            # if that fails try a slower one ...
            logger.info('First attempt to read file failed: ' + \
                        '%s\ntrying a slower approach ....' % msg)

            vals = []

            try:
                for data in [split_initdata, splitdata]:
                    for x in data:
                        try:
                            val = float(x)
                        except Exception:
                            val = NaN
                        vals.append(val)

            except:
                #There is a third slower more robust technique we could
                # implement here.
                logger.exception('Second attempt to read file failed, %s '+ \
                              'Suggest you manually edit the file or ' + \
                              'contact software support.' % file.name)

        # make sure that incomplete lines at the bottom are trimmed
        if len(vals) % column_count > 0:
            vals = vals[:-1*(len(vals)%column_count)]

        # Convert from a List into an array ....
        vals = array(vals)
        try:
            # Reshape the array into a 2d matrix ...
            num_rows = len (vals) / column_count
            vals.shape = (num_rows, column_count)
            # Remove null values ...
            if null_value is not None:
                vals[vals==null_value] = NaN
        except:
            logger.exception("Failed to load well data")
            # probably num_cols giving a divide by zero error
            # or NULL is not defined
            vals = array([])

    except IOError, msg:
        logger.info('Failed to read data from file: ' +  msg)
        vals = array([])

    if len(vals) == 0:
        raise InvalidDataError
    else:
        return vals



def _test():
    import unittest, doctest
    r = unittest.TextTestRunner()
    r.run(doctest.DocTestSuite())

if __name__ == "__main__":
    _test()
